import * as i0 from '@angular/core';
import { Injectable, Component, EventEmitter, Directive, Input, Output, HostBinding, HostListener, ViewChild, ViewEncapsulation, NgModule } from '@angular/core';
import * as i1 from '@angular/common/http';
import { HttpHeaders, HttpClientModule } from '@angular/common/http';
import { of } from 'rxjs';
import { catchError, map } from 'rxjs/operators';
import * as i2 from 'ui-message-angular';
import { messageType } from 'ui-message-angular';
import * as i2$1 from '@angular/forms';
import { FormGroup, FormControl, Validators, FormsModule, ReactiveFormsModule } from '@angular/forms';
import * as i3 from '@angular/common';
import { CommonModule } from '@angular/common';
import { v4 } from 'uuid';

class Entity {
}
class Relationship {
}
class RelationshipInstance {
}
class PartnerInstance {
}
class PartnerRole {
}
class EntityMeta {
}
class Attribute {
}
class Role {
}
class RoleRelation {
}
class RelationshipMeta {
}
class Involve {
}
class RelationMeta {
}
class Association {
}
class FieldsMappingPair {
}
class EntityRelation {
}
class Selection {
}
class Projection {
}
class QueryObject {
}
class Sort {
}
class EntityType {
}
class Relation {
}
class RelationshipH {
}
class RoleH {
}
class RoleMeta {
}
class DataElementH {
}
class DataElementMeta {
}
class DataDomainH {
}
class DataDomainMeta {
}
class DataDomainValue {
}
class SearchHelpH {
}
class SearchHelpMeta {
}
class SearchHelpFieldMeta {
}

class UiMapperService {
    constructor() { }
    /**
     * Map a FormGroup to a JOR relation(with cardinality [0..1] and [1..1]) for adding or updating.
     * @param formGroup: It must be a flat Angular FormGroup, which means no deep structure.
     * @param keys: Business primary keys to identify one row in the relation(DB table).
     * For example: Given {USER_ID: 'DH001'}, it means the relation has the primary key 'USER_ID'.
     * And it must be filled with a fix value 'DH001'.
     * However, if the value of the key can be derived from the formGroup, then assign null value to it.
     * For example: Given {EMAIL: null}, the value of key EMAIL will be derived from formGroup.value['EMAIL'].
     * @param isNew: Determine whether to update or add the value.
     *
     * The return is an object like:
     * {
     *     action: 'update', USER_ID: 'DH001', MIDDLE_NAME: "xxxxxx"
     * }
     */
    composeChangedRelation(formGroup, keys, isNew = true) {
        if (!formGroup.dirty || !keys || Object.keys(keys).length === 0) {
            return {};
        }
        const changedRelation = { action: isNew ? 'add' : 'update' };
        Object.keys(keys).forEach(key => changedRelation[key] = keys[key] || formGroup.value[key]);
        Object.keys(formGroup.controls).forEach(key => {
            const control = formGroup.get(key);
            if (control.dirty) {
                changedRelation[key] = control.value;
            }
        });
        return changedRelation;
    }
    /**
     * Map a FormArray to JOR relation(with cardinality [0..n] or [1..n]) for adding, updating, or deletion.
     * @param formArray: It must be a flat Angular FormArray, which means no deep structure.
     * @param originalArray: To determine whether an item is for adding, or for updating, or for deletion.
     * the original array, which shares the same structure with the formArray, is used for comparison.
     * Tips: You can get the originalArray by calling *FormArray.getRawValue()* right after
     * the FormArray is constructed from the backend data, usually, in ngOnInit().
     * @param keys: Business primary keys to identify one row in the formArray and originalArray.
     * For example: Given {EMAIL: null}, it will use the attribute EMAIL to compare if the same item exists or not.
     * First it iterates the formArray to whether check EMAIL exists in the originalArray,
     * if exists, use action 'update', otherwise, use action 'add'.
     * Then it iterates the originalArray to check EMAIL exists in formArray,
     * if not exists, then the action is set to 'delete'.
     *
     * The return is an array like:
     * [
     *   {action: "update", EMAIL: "dh003@hotmail.com", PRIMARY: 1},
     *   {action: "add", EMAIL: "dh003@gmail.com", TYPE: "work"},
     *   {action: "delete", EMAIL: "dh003@darkhouse.com"}
     * ]
     */
    composeChangedRelationArray(formArray, originalArray, keys) {
        if (!formArray.dirty || !keys || Object.keys(keys).length === 0) {
            return [];
        }
        if (!originalArray) {
            originalArray = [];
        }
        const changedRelationArray = [];
        formArray.controls.forEach(formGroup => {
            if (formGroup.dirty) {
                const changedRelation = {};
                changedRelationArray.push(changedRelation);
                const index = originalArray.findIndex(element => {
                    let found = true;
                    Object.keys(keys).forEach(key => {
                        if (keys[key]) {
                            return;
                        }
                        found = found && (element[key] === formGroup.value[key]);
                    });
                    return found;
                });
                changedRelation['action'] = index === -1 ? 'add' : 'update';
                Object.keys(keys).forEach(key => changedRelation[key] = keys[key] || formGroup.value[key]);
                Object.keys(formGroup['controls']).forEach(key => {
                    const control = formGroup.get(key);
                    if (control.dirty) {
                        changedRelation[key] = control.value;
                    }
                });
            }
        });
        originalArray.forEach(originalRelation => {
            const index = formArray.controls.findIndex(element => {
                let found = true;
                Object.keys(keys).forEach(key => {
                    if (keys[key]) {
                        return;
                    }
                    found = found && (element.value[key] === originalRelation[key]);
                });
                return found;
            });
            if (index === -1) {
                const deletedRelation = { action: 'delete' };
                Object.keys(keys).forEach(key => deletedRelation[key] = keys[key] || originalRelation[key]);
                changedRelationArray.push(deletedRelation);
            }
        });
        return changedRelationArray;
    }
    /**
     * Map a FormArray to JOR relationships for adding, updating, or deletion.
     * @param relationshipID: Relationship ID defined in JOR. For example: 'rs_marriage'
     * @param partnerRoles: Partner roles involved in the relationship.
     * For example: Given [{ENTITY_ID: 'person', ROLE_ID: 'wife'}], it means the partner is a 'person' with role 'wife'.
     * In most cases, a relationship only involves 2 roles, like husband and wife.
     * However, in certain case, there could be more than two. So the partner role is provided as an array.
     * @param formArray: First, it must be a flat Angular FormArray, which means no deep structure.
     * Second, it must contain fields: RELATIONSHIP_INSTANCE_GUID and <partner_role_name>_INSTANCE_GUID.
     * @param originalArray: To determine whether an item is for adding, or for updating, or for deletion.
     * the original array, which shares the same structure with the formArray, is used for comparison.
     * Tips: You can get the originalArray by calling *FormArray.getRawValue()* right after
     * the FormArray is constructed from the backend data, usually, in ngOnInit().
     * @param nonRelationshipAttributes: Attributes that are not belong to the relationship, but exist in the formArray.
     * A relationship has its own attributes, for example, rs_marriage has REG_PLACE and COUNTRY.
     * However, on the UI, you not only want to show the 2 fields, but also the partner's ID, NAME, BIRTHDAY, and so on.
     * And these attributes should be excluded when adding and changing the relationship.
     *
     * The Return will either a null object or a relationship object like:
     * {
     *   RELATIONSHIP_ID: "rs_marriage",
     *   values: [
     *     {action: "add", REG_PLACE: "Shanghai", COUNTRY: "China",
     *      PARTNER_INSTANCES: [
     *       {ENTITY_ID: "person", ROLE_ID: "wife", INSTANCE_GUID: "391E75B02A1811E981F3C33C6FB0A7C1"}
     *     ]},
     *     {action: "update", REG_PLACE: "Beijing", RELATIONSHIP_INSTANCE_GUID: "96DF7F706EE011E9B7B5F7E76DA40E87"},
     *     {action: "delete", RELATIONSHIP_INSTANCE_GUID: "96DF7F706EE011E9B7B5F7E76DA40E87"}
     *   ]
     * }
     */
    composeChangedRelationship(relationshipID, partnerRoles, formArray, originalArray, nonRelationshipAttributes) {
        if (!formArray.dirty) {
            return null;
        }
        const relationship = {
            RELATIONSHIP_ID: relationshipID,
            values: []
        };
        formArray.controls.forEach(formGroup => {
            if (formGroup.dirty) {
                const changedRelationshipValue = {};
                relationship.values.push(changedRelationshipValue);
                if (formGroup.value['RELATIONSHIP_INSTANCE_GUID']) {
                    changedRelationshipValue['action'] = 'update';
                    changedRelationshipValue['RELATIONSHIP_INSTANCE_GUID'] = formGroup.value['RELATIONSHIP_INSTANCE_GUID'];
                }
                else {
                    changedRelationshipValue['action'] = 'add';
                    changedRelationshipValue['PARTNER_INSTANCES'] = [];
                    partnerRoles.forEach(partnerRole => changedRelationshipValue['PARTNER_INSTANCES'].push({
                        ENTITY_ID: partnerRole.ENTITY_ID,
                        ROLE_ID: partnerRole.ROLE_ID,
                        INSTANCE_GUID: formGroup.value[partnerRole.ROLE_ID + '_INSTANCE_GUID']
                    }));
                }
                Object.keys(formGroup['controls']).forEach(key => {
                    if (nonRelationshipAttributes.includes(key) || key.substr(-13) === 'INSTANCE_GUID') {
                        return;
                    }
                    const control = formGroup.get(key);
                    if (control.dirty) {
                        changedRelationshipValue[key] = control.value;
                    }
                });
            }
        });
        originalArray.forEach(original => {
            if (formArray.controls.findIndex(formGroup => formGroup.value['RELATIONSHIP_INSTANCE_GUID'] === original['RELATIONSHIP_INSTANCE_GUID']) === -1) {
                relationship.values.push({ action: 'delete', RELATIONSHIP_INSTANCE_GUID: original['RELATIONSHIP_INSTANCE_GUID'] });
            }
        });
        return relationship.values.length === 0 ? null : relationship;
    }
}
/** @nocollapse */ UiMapperService.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "13.3.9", ngImport: i0, type: UiMapperService, deps: [], target: i0.ɵɵFactoryTarget.Injectable });
/** @nocollapse */ UiMapperService.ɵprov = i0.ɵɵngDeclareInjectable({ minVersion: "12.0.0", version: "13.3.9", ngImport: i0, type: UiMapperService, providedIn: 'root' });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "13.3.9", ngImport: i0, type: UiMapperService, decorators: [{
            type: Injectable,
            args: [{
                    providedIn: 'root'
                }]
        }], ctorParameters: function () { return []; } });

const httpOptions = {
    headers: new HttpHeaders({ 'Content-Type': 'application/json' })
};
class EntityService {
    constructor(http, messageService) {
        this.http = http;
        this.messageService = messageService;
        this.originalHost = '';
        this.fakeUUIDs = [];
        this.messageService.setMessageStore([
            { msgCat: 'EXCEPTION', msgName: 'GENERIC',
                msgText: { EN: { shortText: 'Exception Occurs in Operation: %s', longText: '%s2' } } },
            { msgCat: 'EXCEPTION', msgName: 'SESSION_EXPIRED',
                msgText: { EN: { shortText: 'Your session is expired',
                        longText: 'Your session is expired, please <a href="/logon">re-logon</a>' } } }
        ], 'EN');
    }
    setOriginalHost(host) {
        this.originalHost = host;
    }
    setMessageStore(msgStore, language) {
        this.messageService.setMessageStore(msgStore, language);
    }
    /**
     * List all entity IDs in the system
     */
    listEntityID() {
        return this.http.get(this.originalHost + `/api/entity/EntityIDs`).pipe(catchError(this.handleError('listEntityID')));
    }
    /**
     * List entity IDs by a given role ID
     * @param roleID
     */
    listEntityIDbyRole(roleID) {
        return this.http.get(this.originalHost + `/api/entity/EntityIDs/${roleID}`).pipe(catchError(this.handleError('listEntityIDbyRole')));
    }
    /**
     * Return the entity meta for a given entity ID
     * @param entityID
     */
    getEntityMeta(entityID) {
        return this.http.get(this.originalHost + `/api/entity/meta/${entityID}`).pipe(catchError(this.handleError('getEntityMeta')));
    }
    /**
     * Return a list of entity objects according to the description of a given query object
     * @param queryObject
     */
    searchEntities(queryObject) {
        return this.http.post(this.originalHost + `/api/query`, queryObject, httpOptions).pipe(catchError(this.handleError('searchEntities')));
    }
    /**
     * Return an entity instance for a given entity instance GUID
     * @param instanceGUID
     */
    getEntityInstance(instanceGUID) {
        return this.http.get(this.originalHost + `/api/entity/instance/${instanceGUID}`).pipe(catchError(this.handleError('getEntityInstance')));
    }
    /**
     * Return a relation's meta data for a given relation ID
     * @param relationID
     */
    getRelationMeta(relationID) {
        return this.http.get(this.originalHost + `/api/relation/meta/${relationID}`).pipe(catchError(this.handleError('getRelationMeta')));
    }
    /**
     * Return all relations' meta data for a given entity ID
     * @param entityID
     */
    getRelationMetaOfEntity(entityID) {
        return this.http.get(this.originalHost + `/api/relation/meta/entity/${entityID}`).pipe(catchError(this.handleError('getRelationMetaOfEntity')));
    }
    /**
     * Create a new entity instance
     * @param instance
     */
    createEntityInstance(instance) {
        return this.http.post(this.originalHost + `/api/entity`, instance, httpOptions).pipe(catchError(this.handleError('createEntityInstance')));
    }
    /**
     * Change an existing entity instance
     * @param instance
     */
    changeEntityInstance(instance) {
        return this.http.put(this.originalHost + `/api/entity`, instance, httpOptions).pipe(catchError(this.handleError('changeEntityInstance')));
    }
    /**
     * Delete an entity instance from a given instance GUID
     * @param instanceGUID
     */
    deleteEntityInstance(instanceGUID) {
        return this.http.delete(this.originalHost + `/api/entity/instance/${instanceGUID}`).pipe(catchError(this.handleError('deleteEntityInstance')));
    }
    /**
     * Return entity types(IDs) for a given search term
     * @param term
     */
    listEntityType(term) {
        return this.http.get(this.originalHost + `/api/model/entity-types?term=${term}`).pipe(catchError(this.handleError('listEntityType')));
    }
    /**
     * Return the description of a given entity type(ID)
     * @param entityID
     */
    getEntityTypeDesc(entityID) {
        return this.http.get(this.originalHost + `/api/model/entity-types/${entityID}/desc`).pipe(catchError(this.handleError('getEntityTypeDesc')));
    }
    /**
     * Save an entity type after changing or creation
     * @param entityType
     */
    saveEntityType(entityType) {
        return this.http.post(this.originalHost + `/api/model/entity-types`, entityType, httpOptions).pipe(catchError(this.handleError('saveEntityType')));
    }
    /**
     * Return a list of relations in the system according to the search term
     * @param term
     */
    listRelation(term) {
        return this.http.get(this.originalHost + `/api/model/relations?term=${term}`).pipe(catchError(this.handleError('listRelation')));
    }
    /**
     * Return the description of a given relation ID
     * @param relationID
     */
    getRelationDesc(relationID) {
        return this.http.get(this.originalHost + `/api/model/relations/${relationID}/desc`).pipe(catchError(this.handleError('getRelationDesc')));
    }
    /**
     * Save a relation after changing or creation
     * @param relation
     */
    saveRelation(relation) {
        return this.http.post(this.originalHost + `/api/model/relations`, relation, httpOptions).pipe(catchError(this.handleError('saveRelation')));
    }
    /**
     * Return a list of relationships in the system according to the search term
     * @param term
     */
    listRelationship(term) {
        return this.http.get(this.originalHost + `/api/model/relationships?term=${term}`).pipe(catchError(this.handleError('listRelationship')));
    }
    /**
     * Return a relationship definition from a given relationship ID
     * @param relationshipID
     */
    getRelationship(relationshipID) {
        return this.http.get(this.originalHost + `/api/model/relationships/${relationshipID}`).pipe(catchError(this.handleError('getRelationship')));
    }
    /**
     * Return the description of a given relationship ID
     * @param relationshipID
     */
    getRelationshipDesc(relationshipID) {
        return this.http.get(this.originalHost + `/api/model/relationships/${relationshipID}/desc`).pipe(catchError(this.handleError('getRelationshipDesc')));
    }
    /**
     * Save a relationship after changing or creation
     * @param relationship
     */
    saveRelationship(relationship) {
        return this.http.post(this.originalHost + `/api/model/relationships`, relationship, httpOptions).pipe(catchError(this.handleError('saveRelationship')));
    }
    /**
     * Return a list of roles in the system according to the search term
     * @param term
     */
    listRole(term) {
        return this.http.get(this.originalHost + `/api/model/roles?term=${term}`).pipe(catchError(this.handleError('listRole')));
    }
    /**
     * Return a role definition from a given role ID
     * @param roleID
     */
    getRole(roleID) {
        return this.http.get(this.originalHost + `/api/model/roles/${roleID}`).pipe(catchError(this.handleError('getRole')));
    }
    /**
     * Return the description of a given role ID
     * @param roleID
     */
    getRoleDesc(roleID) {
        return this.http.get(this.originalHost + `/api/model/roles/${roleID}/desc`).pipe(catchError(this.handleError('getRoleDesc')));
    }
    /**
     * Save a role after changing or creation
     * @param role
     */
    saveRole(role) {
        return this.http.post(this.originalHost + `/api/model/roles`, role, httpOptions).pipe(catchError(this.handleError('saveRole')));
    }
    /**
     * Return a list of data elements in the system according to the search term
     * @param term
     */
    listDataElement(term) {
        return this.http.get(this.originalHost + `/api/model/data-elements?term=${term}`).pipe(catchError(this.handleError('listDataElement')));
    }
    /**
     * Return a data element definition from a given element ID
     * @param elementID
     */
    getDataElement(elementID) {
        return this.http.get(this.originalHost + `/api/model/data-elements/${elementID}`).pipe(catchError(this.handleError('getDataElement')));
    }
    /**
     * Return the description of a given data element ID
     * @param elementID
     */
    getDataElementDesc(elementID) {
        return this.http.get(this.originalHost + `/api/model/data-elements/${elementID}/desc`).pipe(catchError(this.handleError('getDataElementDesc')));
    }
    /**
     * Save a data element after changing or creation
     * @param element
     */
    saveDataElement(element) {
        return this.http.post(this.originalHost + `/api/model/data-elements`, element, httpOptions).pipe(catchError(this.handleError('saveDataElement')));
    }
    /**
     * Return a list of data domains in the system according to the search term
     * @param term
     */
    listDataDomain(term) {
        return this.http.get(this.originalHost + `/api/model/data-domains?term=${term}`).pipe(catchError(this.handleError('listDataDomain')));
    }
    /**
     * Return a data domain definition from a given domain ID
     * @param domainID
     */
    getDataDomain(domainID) {
        return this.http.get(this.originalHost + `/api/model/data-domains/${domainID}`).pipe(catchError(this.handleError('getDataDomain')));
    }
    /**
     * Return the description of a given data domain ID
     * @param domainID
     */
    getDataDomainDesc(domainID) {
        return this.http.get(this.originalHost + `/api/model/data-domains/${domainID}/desc`).pipe(catchError(this.handleError('getDataDomainDesc')));
    }
    /**
     * Save a data domain after changing or creation
     * @param domain
     */
    saveDataDomain(domain) {
        return this.http.post(this.originalHost + `/api/model/data-domains`, domain, httpOptions).pipe(catchError(this.handleError('saveDataDomain')));
    }
    /**
     * Return a list of search helps in the system according to the search term
     * @param term
     */
    listSearchHelp(term) {
        return this.http.get(this.originalHost + `/api/model/search-helps?term=${term}`).pipe(catchError(this.handleError('listSearchHelp')));
    }
    /**
     * Return a search help definition from a given domain ID
     * @param searchHelpID
     */
    getSearchHelp(searchHelpID) {
        return this.http.get(this.originalHost + `/api/model/search-helps/${searchHelpID}`).pipe(catchError(this.handleError('getSearchHelp')));
    }
    /**
     * Return the description of a given search help ID
     * @param searchHelpID
     */
    getSearchHelpDesc(searchHelpID) {
        return this.http.get(this.originalHost + `/api/model/search-helps/${searchHelpID}/desc`).pipe(catchError(this.handleError('getSearchHelpDesc')));
    }
    /**
     * Save a search help after changing or creation
     * @param searchHelp
     */
    saveSearchHelp(searchHelp) {
        return this.http.post(this.originalHost + `/api/model/search-helps`, searchHelp, httpOptions).pipe(catchError(this.handleError('saveSearchHelp')));
    }
    /**
     * Get data element meta
     * @param elementID
     */
    getElementMeta(elementID) {
        return this.http.get(this.originalHost + `/api/model/element-meta/${elementID}`).pipe(catchError(this.handleError('getElementMeta')));
    }
    /**
     * Return a fake relationship ID before saving
     */
    generateFakeRelationshipUUID() {
        const nextPosition = this.fakeUUIDs.length + 1;
        const fakeUUID = 'NewRelationship_' + nextPosition;
        this.fakeUUIDs.push(fakeUUID);
        return fakeUUID;
    }
    handleError(operation = 'operation', result) {
        return (error) => {
            // TODO: send the error to remote logging infrastructure
            console.error(error); // log to console instead
            if (error.status === 401) {
                this.messageService.addMessage('EXCEPTION', 'SESSION_EXPIRED', messageType.Exception);
            }
            else {
                this.messageService.addMessage('EXCEPTION', 'GENERIC', messageType.Exception, operation, error.message);
            }
            // Let the app keep running by returning an empty result.
            return of(result);
        };
    }
}
/** @nocollapse */ EntityService.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "13.3.9", ngImport: i0, type: EntityService, deps: [{ token: i1.HttpClient }, { token: i2.MessageService }], target: i0.ɵɵFactoryTarget.Injectable });
/** @nocollapse */ EntityService.ɵprov = i0.ɵɵngDeclareInjectable({ minVersion: "12.0.0", version: "13.3.9", ngImport: i0, type: EntityService, providedIn: 'root' });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "13.3.9", ngImport: i0, type: EntityService, decorators: [{
            type: Injectable,
            args: [{
                    providedIn: 'root'
                }]
        }], ctorParameters: function () { return [{ type: i1.HttpClient }, { type: i2.MessageService }]; } });

class SearchHelp {
}
class SearchHelpField {
}

class SearchHelpComponent {
    constructor(fb, entityService) {
        this.fb = fb;
        this.entityService = entityService;
        this.isSearchHelpModalShown = false;
        this.isFilterShown = false;
        this.isSelectAllChecked = false;
        this.selectedIndex = -1;
        this.filterFields = [];
        this.listFields = [];
        this.listData = [];
        this.preSearchHelpParas = {};
    }
    get displaySearchHelpModal() { return this.isSearchHelpModalShown ? 'block' : 'none'; }
    ngOnInit() {
        this.filterFieldsFormGroup = this.fb.group({});
    }
    /**
     * Open an search help dialog with free style
     * @param searchHelp
     * @param exportControl: An Angular form control which is used to receive the Search Help returned value.
     * @param afterExportFn: Optional. If provided, the function will be executed after the value is returned.
     */
    openSearchHelpModal(searchHelp, exportControl, afterExportFn) {
        if (this.searchHelp !== searchHelp) {
            this.listData = [];
            this.searchHelp = searchHelp;
            this.filterFieldsFormGroup = this.fb.group({});
            this.filterFields = this.searchHelp.FIELDS.filter(fieldMeta => fieldMeta.FILTER_POSITION);
            this.filterFields.sort((a, b) => a.FILTER_POSITION - b.FILTER_POSITION);
            this.filterFields.forEach(fieldMeta => {
                if (fieldMeta.IMPORT) {
                    const ieFieldName = fieldMeta.IE_FIELD_NAME || fieldMeta.FIELD_NAME;
                    if (exportControl.get(ieFieldName)) {
                        fieldMeta.DEFAULT_VALUE = exportControl.get(ieFieldName).value;
                    }
                }
                this.filterFieldsFormGroup.addControl(fieldMeta.FIELD_NAME, this.fb.control(fieldMeta.DEFAULT_VALUE));
            });
            this.listFields = this.searchHelp.FIELDS.filter(fieldMeta => fieldMeta.LIST_POSITION);
            this.listFields.sort((a, b) => a.LIST_POSITION - b.LIST_POSITION);
        }
        // Import the value to fuzzy search field
        if (this.searchHelp.FUZZY_SEARCH) {
            this.fuzzySearchTerm = '';
            this.searchHelp.FIELDS.forEach(fieldMeta => {
                const ieFieldName = fieldMeta.IE_FIELD_NAME || fieldMeta.FIELD_NAME;
                const ctrl = exportControl.get(ieFieldName);
                if (!this.fuzzySearchTerm && ctrl && ctrl.value) {
                    this.fuzzySearchTerm = ctrl.value;
                    return;
                }
            });
        }
        this.exportControl = exportControl;
        this.afterExportFn = afterExportFn;
        if (this.searchHelp.BEHAVIOUR === 'A') {
            this.search();
            this.isFilterShown = false;
        }
        else {
            this.isFilterShown = true;
        }
        this.isSearchHelpModalShown = true;
    }
    /**
     * Open a search help dialog based on the given entity and one of its relation
     * @param entityID
     * @param relationID
     * @param exportControl: An Angular form control which is used to receive the Search Help returned value.
     * @param readonly: If readonly, then the Search Help cannot return value.
     * @param exportField: Provided only if exportField is given. It uses the data domain to find which attribute should return the value(s).
     For example, attribute "USER" is assigned with Data Domain "USER_ID". When the Search Help dialog pops up on the field 'CREATE_BY',
     it finds the attribute "USER" using the Data Domain "USER_ID",
     and the value of its attribute "USER" is then exported to the field "CREATE_BY".
     * @param afterExportFn: Optional. If provided, the function will be executed after the value is returned.
     */
    openSearchHelpModalByEntity(entityID, relationID, exportControl, readonly, exportField, domainID, afterExportFn) {
        const currentSearchHelpParas = {
            'entityID': entityID,
            'relationID': relationID,
            'exportField': exportField,
            'domainID': domainID
        };
        if (this._isPreviousSearchHelp(currentSearchHelpParas)) {
            this.searchHelp.READ_ONLY = readonly;
            this.openSearchHelpModal(this.searchHelp, exportControl, afterExportFn);
        }
        else {
            this.listData = [];
            this.preSearchHelpParas = currentSearchHelpParas;
            const searchHelp = new SearchHelp();
            searchHelp.OBJECT_NAME = entityID;
            searchHelp.METHOD = function (entityService) {
                return (searchTerm) => entityService.searchEntities(searchTerm);
            }(this.entityService);
            searchHelp.BEHAVIOUR = 'M';
            searchHelp.MULTI = false;
            searchHelp.FUZZY_SEARCH = false;
            searchHelp.FIELDS = [];
            searchHelp.READ_ONLY = readonly;
            searchHelp.ENTITY_ID = entityID;
            searchHelp.RELATION_ID = relationID;
            this.entityService.getRelationMeta(relationID)
                .subscribe(data => {
                const relationMeta = data;
                relationMeta.ATTRIBUTES.forEach(attribute => searchHelp.FIELDS.push({
                    FIELD_NAME: attribute.ATTR_NAME,
                    LABEL_TEXT: attribute.LABEL_TEXT,
                    LIST_HEADER_TEXT: attribute.LIST_HEADER_TEXT,
                    IE_FIELD_NAME: exportField && domainID && domainID === attribute.DOMAIN_ID ? exportField : null,
                    IMPORT: attribute.PRIMARY_KEY || attribute.DOMAIN_ID === domainID,
                    EXPORT: attribute.PRIMARY_KEY || attribute.DOMAIN_ID === domainID,
                    LIST_POSITION: attribute.ORDER,
                    FILTER_POSITION: attribute.ORDER
                }));
                searchHelp.FIELDS.push({
                    FIELD_NAME: 'INSTANCE_GUID',
                    LIST_HEADER_TEXT: 'GUID',
                    IMPORT: false,
                    EXPORT: true,
                    LIST_POSITION: 999,
                    FILTER_POSITION: 0
                });
                this.openSearchHelpModal(searchHelp, exportControl, afterExportFn);
            });
        }
    }
    /**
     * Open a search help dialog based on the given search help
     * @param searchHelpID
     * @param searchHelpExportField: a field name in the Search Help which is tagged as exported.
     An Search Help can have multiple exported fields, and the field names may be different with the Angular control names.
     Thus, you can choose one Search Help export field name to map with one Angular field control name.
     * @param exportField: An Angular field control name that is to map with the Search Help export field name.
     * @param exportControl: An Angular form control which is used to receive the Search Help returned value.
     * @param readonly: If readonly, then the Search Help cannot return value.
     * @param afterExportFn: Optional. If provided, the function will be executed after the value is returned.
     */
    openSearchHelpModalBySearchHelp(searchHelpID, searchHelpExportField, exportField, exportControl, readonly, afterExportFn) {
        const currentSearchHelpParas = {
            'searchHelpID': searchHelpID,
            'searchHelpExportField': searchHelpExportField,
            'exportField': exportField
        };
        if (this._isPreviousSearchHelp(currentSearchHelpParas)) {
            this.searchHelp.READ_ONLY = readonly;
            this.openSearchHelpModal(this.searchHelp, exportControl, afterExportFn);
        }
        else {
            this.listData = [];
            this.preSearchHelpParas = currentSearchHelpParas;
            const searchHelp = new SearchHelp();
            this.entityService.getSearchHelp(searchHelpID)
                .subscribe((searchHelpMeta) => {
                searchHelp.OBJECT_NAME = searchHelpMeta.SEARCH_HELP_DESC + '(' + searchHelpMeta.SEARCH_HELP_ID + ')';
                searchHelp.METHOD = function (entityService) {
                    return (searchTerm) => entityService.searchEntities(searchTerm);
                }(this.entityService);
                searchHelp.BEHAVIOUR = searchHelpMeta.BEHAVIOUR;
                searchHelp.MULTI = searchHelpMeta.MULTI;
                searchHelp.FUZZY_SEARCH = searchHelpMeta.FUZZY_SEARCH;
                searchHelp.READ_ONLY = readonly;
                searchHelp.ENTITY_ID = searchHelpMeta.ENTITY_ID;
                searchHelp.RELATION_ID = searchHelpMeta.RELATION_ID;
                searchHelp.FIELDS = searchHelpMeta.FIELDS;
                const searchHelpField = searchHelp.FIELDS.find(field => (field.IE_FIELD_NAME || field.FIELD_NAME) === searchHelpExportField);
                searchHelpField.IE_FIELD_NAME = exportField;
                searchHelpField.EXPORT = true;
                searchHelpField.IMPORT = true;
                searchHelp.FIELDS.push({
                    FIELD_NAME: 'INSTANCE_GUID',
                    LIST_HEADER_TEXT: 'GUID',
                    IMPORT: false,
                    EXPORT: true,
                    LIST_POSITION: 999,
                    FILTER_POSITION: 0
                });
                this.openSearchHelpModal(searchHelp, exportControl, afterExportFn);
            });
        }
    }
    _isPreviousSearchHelp(currentSearchHelpParas) {
        return JSON.stringify(this.preSearchHelpParas) === JSON.stringify(currentSearchHelpParas);
    }
    search() {
        let searchTerm;
        if (this.searchHelp.FUZZY_SEARCH) {
            searchTerm = this.fuzzySearchTerm || '';
        }
        else {
            searchTerm = new QueryObject();
            searchTerm.ENTITY_ID = this.searchHelp.ENTITY_ID;
            searchTerm.RELATION_ID = this.searchHelp.RELATION_ID;
            searchTerm.FILTER = [];
            this.filterFields.forEach(fieldMeta => {
                const fieldValue = this.filterFieldsFormGroup.get(fieldMeta.FIELD_NAME).value;
                if (fieldValue) {
                    if (fieldValue.includes('*') || fieldValue.includes('%')) {
                        searchTerm.FILTER.push({ RELATION_ID: fieldMeta.RELATION_ID,
                            FIELD_NAME: fieldMeta.FIELD_NAME, OPERATOR: 'CN', LOW: fieldValue });
                    }
                    else {
                        searchTerm.FILTER.push({ RELATION_ID: fieldMeta.RELATION_ID,
                            FIELD_NAME: fieldMeta.FIELD_NAME, OPERATOR: 'EQ', LOW: fieldValue });
                    }
                }
            });
            searchTerm.PROJECTION = [];
            this.listFields.forEach(fieldMeta => {
                searchTerm.PROJECTION.push({ RELATION_ID: fieldMeta.RELATION_ID, FIELD_NAME: fieldMeta.FIELD_NAME });
            });
        }
        this.listData = [];
        this.isSelectAllChecked = false;
        if (typeof this.searchHelp.METHOD === 'function') {
            const searchHelpMethod = this.searchHelp.METHOD;
            searchHelpMethod(searchTerm).subscribe(data => this._generateSearchList(data));
        }
        else if (Array.isArray(this.searchHelp.METHOD)) {
            this._generateSearchList(this.searchHelp.METHOD);
        }
    }
    _generateSearchList(data) {
        data.forEach(item => {
            const listItem = { SELECTED: '' };
            this.listFields.forEach(field => listItem[field.FIELD_NAME] = item[field.FIELD_NAME]);
            this.listData.push(listItem);
        });
    }
    enterSearch($event) {
        if ($event.keyCode === 13) {
            this.search();
        }
    }
    showFilter() {
        this.isFilterShown = true;
    }
    hideFilter() {
        this.isFilterShown = false;
    }
    selectAll() {
        this.isSelectAllChecked = !this.isSelectAllChecked;
        this.listData.forEach(data => data.SELECTED = !data.SELECTED);
    }
    confirmSelection() {
        // TODO: Currently, only single selection is supported. Multiple selection in later time
        if (this.searchHelp.READ_ONLY) {
            return;
        }
        this.listFields.forEach(listField => {
            if (this.exportControl instanceof FormGroup) {
                const exportControl = this.exportControl;
                const ieFieldName = listField.IE_FIELD_NAME || listField.FIELD_NAME;
                const exportFieldControl = exportControl.get(ieFieldName);
                if (listField.EXPORT && exportFieldControl) {
                    exportFieldControl.setValue(this.listData[this.selectedIndex][listField.FIELD_NAME]);
                    exportFieldControl.markAsDirty();
                }
            }
            else {
                if (listField.EXPORT) {
                    this.exportControl[listField.FIELD_NAME] = this.listData[this.selectedIndex][listField.FIELD_NAME];
                }
            }
        });
        if (this.afterExportFn) {
            this.afterExportFn();
        }
        this.listData = [];
        this.isSearchHelpModalShown = false;
    }
    closeSearchHelpModal() {
        this.isSearchHelpModalShown = false;
    }
}
/** @nocollapse */ SearchHelpComponent.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "13.3.9", ngImport: i0, type: SearchHelpComponent, deps: [{ token: i2$1.FormBuilder }, { token: EntityService }], target: i0.ɵɵFactoryTarget.Component });
/** @nocollapse */ SearchHelpComponent.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "12.0.0", version: "13.3.9", type: SearchHelpComponent, selector: "dk-app-search-help", ngImport: i0, template: "<div class=\"modal fade\" id=\"searchHelp\" tabindex=\"-1\" role=\"dialog\"\n     [ngClass]=\"{'show': isSearchHelpModalShown}\" [ngStyle]=\"{'display': displaySearchHelpModal}\">\n  <div class=\"modal-dialog modal-dialog-centered modal-lg\" role=\"document\">\n    <div class=\"modal-content dk-search-help-content\">\n      <div class=\"modal-header dk-search-help-header\">\n        <h6 class=\"modal-title\" id=\"addRelationship\">{{searchHelp?.OBJECT_NAME}} ({{listData.length}})</h6>\n        <button type=\"button\" class=\"close\" (click)=\"closeSearchHelpModal()\">\n          <span aria-hidden=\"true\">&times;</span>\n        </button>\n      </div>\n\n      <div class=\"modal-body\">\n        <div *ngIf=\"isFilterShown\" class=\"card mb-2\">\n          <div *ngIf=\"searchHelp && searchHelp.FUZZY_SEARCH\" class=\"mt-2 mx-2 row\">\n            <input type=\"text\" class=\"form-control\" placeholder=\"Search\" [(ngModel)]=\"fuzzySearchTerm\" (keyup)=\"enterSearch($event)\">\n          </div>\n          <div class=\"mt-2 mx-2 row\" [formGroup]=\"filterFieldsFormGroup\">\n            <div *ngFor=\"let filterField of filterFields\" class=\"col-6 form-group row\">\n              <label for=\"{{filterField.FIELD_NAME}}\" class=\"col-6 col-form-label col-form-label-sm text-right\">\n                {{filterField.LABEL_TEXT || filterField.FIELD_NAME}}:</label>\n              <input id=\"{{filterField.FIELD_NAME}}\" name=\"{{filterField.FIELD_NAME}}\" formControlName=\"{{filterField.FIELD_NAME}}\"\n                     type=\"text\" class=\"col-6 form-control form-control-sm\" [readonly]=\"filterField.FILTER_DISP_ONLY\" (keyup)=\"enterSearch($event)\">\n            </div>\n          </div>\n\n          <div class=\"mb-2\">\n            <button type=\"button\" id=\"search\" class=\"btn btn-sm btn-primary float-right mr-2\" (click)=\"search()\">\n              Search\n            </button>\n          </div>\n        </div>\n\n        <button *ngIf=\"!isFilterShown\" type=\"button\" class=\"btn btn-link float-right\" (click)=\"showFilter()\">Show Filter</button>\n        <button *ngIf=\"isFilterShown\" type=\"button\" class=\"btn btn-link float-right\" (click)=\"hideFilter()\">Hide Filter</button>\n\n        <table class=\"table table-bordered table-sm table-hover dk-search-table\">\n          <thead class=\"thead-light\">\n          <tr>\n            <th scope=\"col\">\n              <input *ngIf=\"searchHelp?.MULTI\" type=\"checkbox\" [(ngModel)]=\"isSelectAllChecked\" (click)=\"selectAll()\">\n            </th>\n            <th *ngFor=\"let listField of listFields\"\n                [style.display]=\"listField.FIELD_NAME === 'INSTANCE_GUID' ? 'none' : 'table-cell'\">\n              {{listField.LIST_HEADER_TEXT || listField.FIELD_NAME}}\n            </th>\n          </tr>\n          </thead>\n\n          <tbody>\n          <tr *ngFor=\"let item of listData; let i = index\">\n            <td>\n              <input *ngIf=\"!searchHelp.MULTI\" type=\"radio\" name=\"selectedIndex\" [value]=\"i\" [(ngModel)]=\"selectedIndex\" (dblclick)=\"confirmSelection()\">\n              <input *ngIf=\"searchHelp.MULTI\" type=\"checkbox\" [(ngModel)]=\"item.SELECTED\">\n            </td>\n            <td *ngFor=\"let listField of listFields\"\n                [style.display]=\"listField.FIELD_NAME === 'INSTANCE_GUID' ? 'none' : 'table-cell'\">\n              {{item[listField.FIELD_NAME]}}\n            </td>\n          </tr>\n          </tbody>\n        </table>\n      </div>\n\n      <div class=\"modal-footer dk-search-help-footer\">\n        <button type=\"button\" class=\"btn btn-sm btn-secondary\" (click)=\"closeSearchHelpModal()\">Cancel</button>\n        <button type=\"button\" class=\"btn btn-sm btn-primary\" (click)=\"confirmSelection()\" [disabled]=\"searchHelp?.READ_ONLY\">Confirm</button>\n      </div>\n    </div>\n  </div>\n</div>\n", styles: [".dk-search-help-header{padding:.3rem .5rem;background-color:#e9ecef}.dk-search-help-footer{padding:.5rem}.dk-search-table{display:block;max-height:20rem;overflow:scroll;white-space:nowrap;border:none}.dk-search-help-content{border:1px solid rgba(0,0,0,.4);box-shadow:8px 5px 5px #0003}.dk-search-table th{position:sticky;top:0;z-index:2}\n"], directives: [{ type: i3.NgClass, selector: "[ngClass]", inputs: ["class", "ngClass"] }, { type: i3.NgStyle, selector: "[ngStyle]", inputs: ["ngStyle"] }, { type: i3.NgIf, selector: "[ngIf]", inputs: ["ngIf", "ngIfThen", "ngIfElse"] }, { type: i2$1.DefaultValueAccessor, selector: "input:not([type=checkbox])[formControlName],textarea[formControlName],input:not([type=checkbox])[formControl],textarea[formControl],input:not([type=checkbox])[ngModel],textarea[ngModel],[ngDefaultControl]" }, { type: i2$1.NgControlStatus, selector: "[formControlName],[ngModel],[formControl]" }, { type: i2$1.NgModel, selector: "[ngModel]:not([formControlName]):not([formControl])", inputs: ["name", "disabled", "ngModel", "ngModelOptions"], outputs: ["ngModelChange"], exportAs: ["ngModel"] }, { type: i2$1.NgControlStatusGroup, selector: "[formGroupName],[formArrayName],[ngModelGroup],[formGroup],form:not([ngNoForm]),[ngForm]" }, { type: i2$1.FormGroupDirective, selector: "[formGroup]", inputs: ["formGroup"], outputs: ["ngSubmit"], exportAs: ["ngForm"] }, { type: i3.NgForOf, selector: "[ngFor][ngForOf]", inputs: ["ngForOf", "ngForTrackBy", "ngForTemplate"] }, { type: i2$1.FormControlName, selector: "[formControlName]", inputs: ["formControlName", "disabled", "ngModel"], outputs: ["ngModelChange"] }, { type: i2$1.CheckboxControlValueAccessor, selector: "input[type=checkbox][formControlName],input[type=checkbox][formControl],input[type=checkbox][ngModel]" }, { type: i2$1.RadioControlValueAccessor, selector: "input[type=radio][formControlName],input[type=radio][formControl],input[type=radio][ngModel]", inputs: ["name", "formControlName", "value"] }] });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "13.3.9", ngImport: i0, type: SearchHelpComponent, decorators: [{
            type: Component,
            args: [{ selector: 'dk-app-search-help', template: "<div class=\"modal fade\" id=\"searchHelp\" tabindex=\"-1\" role=\"dialog\"\n     [ngClass]=\"{'show': isSearchHelpModalShown}\" [ngStyle]=\"{'display': displaySearchHelpModal}\">\n  <div class=\"modal-dialog modal-dialog-centered modal-lg\" role=\"document\">\n    <div class=\"modal-content dk-search-help-content\">\n      <div class=\"modal-header dk-search-help-header\">\n        <h6 class=\"modal-title\" id=\"addRelationship\">{{searchHelp?.OBJECT_NAME}} ({{listData.length}})</h6>\n        <button type=\"button\" class=\"close\" (click)=\"closeSearchHelpModal()\">\n          <span aria-hidden=\"true\">&times;</span>\n        </button>\n      </div>\n\n      <div class=\"modal-body\">\n        <div *ngIf=\"isFilterShown\" class=\"card mb-2\">\n          <div *ngIf=\"searchHelp && searchHelp.FUZZY_SEARCH\" class=\"mt-2 mx-2 row\">\n            <input type=\"text\" class=\"form-control\" placeholder=\"Search\" [(ngModel)]=\"fuzzySearchTerm\" (keyup)=\"enterSearch($event)\">\n          </div>\n          <div class=\"mt-2 mx-2 row\" [formGroup]=\"filterFieldsFormGroup\">\n            <div *ngFor=\"let filterField of filterFields\" class=\"col-6 form-group row\">\n              <label for=\"{{filterField.FIELD_NAME}}\" class=\"col-6 col-form-label col-form-label-sm text-right\">\n                {{filterField.LABEL_TEXT || filterField.FIELD_NAME}}:</label>\n              <input id=\"{{filterField.FIELD_NAME}}\" name=\"{{filterField.FIELD_NAME}}\" formControlName=\"{{filterField.FIELD_NAME}}\"\n                     type=\"text\" class=\"col-6 form-control form-control-sm\" [readonly]=\"filterField.FILTER_DISP_ONLY\" (keyup)=\"enterSearch($event)\">\n            </div>\n          </div>\n\n          <div class=\"mb-2\">\n            <button type=\"button\" id=\"search\" class=\"btn btn-sm btn-primary float-right mr-2\" (click)=\"search()\">\n              Search\n            </button>\n          </div>\n        </div>\n\n        <button *ngIf=\"!isFilterShown\" type=\"button\" class=\"btn btn-link float-right\" (click)=\"showFilter()\">Show Filter</button>\n        <button *ngIf=\"isFilterShown\" type=\"button\" class=\"btn btn-link float-right\" (click)=\"hideFilter()\">Hide Filter</button>\n\n        <table class=\"table table-bordered table-sm table-hover dk-search-table\">\n          <thead class=\"thead-light\">\n          <tr>\n            <th scope=\"col\">\n              <input *ngIf=\"searchHelp?.MULTI\" type=\"checkbox\" [(ngModel)]=\"isSelectAllChecked\" (click)=\"selectAll()\">\n            </th>\n            <th *ngFor=\"let listField of listFields\"\n                [style.display]=\"listField.FIELD_NAME === 'INSTANCE_GUID' ? 'none' : 'table-cell'\">\n              {{listField.LIST_HEADER_TEXT || listField.FIELD_NAME}}\n            </th>\n          </tr>\n          </thead>\n\n          <tbody>\n          <tr *ngFor=\"let item of listData; let i = index\">\n            <td>\n              <input *ngIf=\"!searchHelp.MULTI\" type=\"radio\" name=\"selectedIndex\" [value]=\"i\" [(ngModel)]=\"selectedIndex\" (dblclick)=\"confirmSelection()\">\n              <input *ngIf=\"searchHelp.MULTI\" type=\"checkbox\" [(ngModel)]=\"item.SELECTED\">\n            </td>\n            <td *ngFor=\"let listField of listFields\"\n                [style.display]=\"listField.FIELD_NAME === 'INSTANCE_GUID' ? 'none' : 'table-cell'\">\n              {{item[listField.FIELD_NAME]}}\n            </td>\n          </tr>\n          </tbody>\n        </table>\n      </div>\n\n      <div class=\"modal-footer dk-search-help-footer\">\n        <button type=\"button\" class=\"btn btn-sm btn-secondary\" (click)=\"closeSearchHelpModal()\">Cancel</button>\n        <button type=\"button\" class=\"btn btn-sm btn-primary\" (click)=\"confirmSelection()\" [disabled]=\"searchHelp?.READ_ONLY\">Confirm</button>\n      </div>\n    </div>\n  </div>\n</div>\n", styles: [".dk-search-help-header{padding:.3rem .5rem;background-color:#e9ecef}.dk-search-help-footer{padding:.5rem}.dk-search-table{display:block;max-height:20rem;overflow:scroll;white-space:nowrap;border:none}.dk-search-help-content{border:1px solid rgba(0,0,0,.4);box-shadow:8px 5px 5px #0003}.dk-search-table th{position:sticky;top:0;z-index:2}\n"] }]
        }], ctorParameters: function () { return [{ type: i2$1.FormBuilder }, { type: EntityService }]; } });

class AttributeBase {
    constructor() {
        this.dropdownList = [];
    }
}
class DropdownList {
}

// noinspection JSAnnotator
class DomainValueValidator {
    constructor(entityService) {
        this.entityService = entityService;
    }
    validate(ctrl) {
        const queryObject = new QueryObject();
        // queryObject.ENTITY_ID = entityID;
        // queryObject.RELATION_ID = relationID;
        // queryObject.FILTER = [{FIELD_NAME: searchField, OPERATOR: 'EQ', LOW: ctrl.value}];
        return this.entityService.searchEntities(queryObject).pipe(map(data => {
            if (data['msgName']) {
                return { message: data['msgName']['msgShortText'] };
            }
            else {
                if (data.length === 0) {
                    return { message: 'Value is invalid' };
                }
                else {
                    return null;
                }
            }
        }), catchError(() => null));
    }
}
/** @nocollapse */ DomainValueValidator.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "13.3.9", ngImport: i0, type: DomainValueValidator, deps: [{ token: EntityService }], target: i0.ɵɵFactoryTarget.Injectable });
/** @nocollapse */ DomainValueValidator.ɵprov = i0.ɵɵngDeclareInjectable({ minVersion: "12.0.0", version: "13.3.9", ngImport: i0, type: DomainValueValidator, providedIn: 'root' });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "13.3.9", ngImport: i0, type: DomainValueValidator, decorators: [{
            type: Injectable,
            args: [{
                    providedIn: 'root'
                }]
        }], ctorParameters: function () { return [{ type: EntityService }]; } });

class AttributeControlService {
    constructor(entityService, domainValueValidator) {
        this.entityService = entityService;
        this.domainValueValidator = domainValueValidator;
    }
    toAttributeControl(attributes) {
        const attributeControls = [];
        if (!attributes) {
            return attributeControls;
        }
        attributes.forEach(attribute => attributeControls.push(this.toSingleAttributeControl(attribute)));
        return attributeControls;
    }
    toSingleAttributeControl(attribute) {
        const attributeControl = new AttributeBase();
        attributeControl.key = attribute.ATTR_GUID;
        attributeControl.name = attribute.ATTR_NAME;
        attributeControl.label = attribute.LABEL_TEXT;
        attributeControl.list_label = attribute.LIST_HEADER_TEXT;
        attributeControl.relationId = attribute.RELATION_ID;
        attributeControl.searchHelpId = attribute.SEARCH_HELP_ID;
        attributeControl.searchHelpExportField = attribute.SEARCH_HELP_EXPORT_FIELD;
        attributeControl.domainId = attribute.DOMAIN_ID;
        attributeControl.domainEntityId = attribute.DOMAIN_ENTITY_ID;
        attributeControl.domainRelationId = attribute.DOMAIN_RELATION_ID;
        switch (attribute.DATA_TYPE) {
            case 1: // Char
                if (attribute.CAPITAL_ONLY) {
                    attributeControl.controlType = 'text_capital';
                    // onkeyup="this.value = this.value.toUpperCase();"
                }
                else {
                    if (attribute.DOMAIN_TYPE === 3) {
                        attributeControl.controlType = 'dropdown';
                        this._generateDropdownList(attribute.DOMAIN_ID, attributeControl);
                    }
                    else {
                        attributeControl.controlType = 'text';
                        attributeControl.pattern = attribute.REG_EXPR;
                    }
                }
                attributeControl.maxLength = attribute.DATA_LENGTH;
                attributeControl.primaryKey = attribute.PRIMARY_KEY;
                break;
            case 2: // Integer
                if (attribute.DOMAIN_TYPE === 3) {
                    attributeControl.controlType = 'dropdown';
                    this._generateDropdownList(attribute.DOMAIN_ID, attributeControl);
                }
                else {
                    attributeControl.controlType = 'integer';
                    if (attribute.UNSIGNED) {
                        attributeControl.pattern = '^\\d+([^.,])?$';
                    }
                    attributeControl.autoIncrement = attribute.AUTO_INCREMENT;
                }
                attributeControl.primaryKey = attribute.PRIMARY_KEY;
                break;
            case 3: // Boolean
                attributeControl.controlType = 'checkbox';
                break;
            case 4: // Decimal
                attributeControl.controlType = 'decimal';
                this._setDecimalPattern(attributeControl, attribute);
                break;
            case 5: // String
                attributeControl.controlType = 'textarea';
                break;
            case 6: // Binary
                attributeControl.controlType = 'file';
                break;
            case 7: // Date
                attributeControl.controlType = 'date';
                break;
            case 8: // Timestamp
                attributeControl.controlType = 'timestamp';
                break;
            default:
                attributeControl.controlType = 'text';
        }
        return attributeControl;
    }
    _generateDropdownList(domainID, attributeControl) {
        this.entityService.getDataDomain(domainID)
            .subscribe(dataDomain => dataDomain['DOMAIN_VALUES'].forEach(domainValue => {
            attributeControl.dropdownList.push({
                key: domainValue['LOW_VALUE'],
                value: domainValue['LOW_VALUE_TEXT'] || domainValue['LOW_VALUE']
            });
        }));
    }
    _setDecimalPattern(attributeControl, attribute) {
        const zeroPadding = '0000000000000000000000000000000000000';
        attributeControl.step = '0.' + zeroPadding.substr(0, attribute.DECIMAL - 1) + '1';
        attributeControl.placeholder = '0.' + zeroPadding.substr(0, attribute.DECIMAL - 1) + '0';
        const integerPlace = attribute.DATA_LENGTH - attribute.DECIMAL;
        attributeControl.pattern = attribute.UNSIGNED ? '^(' : '^(\\-?';
        attributeControl.pattern += integerPlace ? '\\d{1,' + integerPlace.toString() + '})' : '0)';
        attributeControl.pattern += '(\\.\\d{1,' + attribute.DECIMAL + '})?$';
        attributeControl.maxLength = attribute.DATA_LENGTH;
    }
    convertToFormControl(attribute, instance) {
        const formControl = new FormControl(instance[attribute.ATTR_NAME] || '');
        if (attribute.PRIMARY_KEY && !attribute.AUTO_INCREMENT) {
            formControl.setValidators(Validators.required);
        }
        // if (attribute.DOMAIN_TYPE === 2 && !attribute.PRIMARY_KEY) {
        //   formControl.setAsyncValidators(this.domainValueValidator.validate.bind(this.domainValueValidator));
        // }
        return formControl;
    }
    convertToFormGroup(attributes, instance, isDirty) {
        const group = {};
        attributes.forEach(attribute => {
            group[attribute.ATTR_NAME] = this.convertToFormControl(attribute, instance);
            if (instance[attribute.ATTR_NAME] && isDirty) {
                group[attribute.ATTR_NAME].markAsDirty();
            }
        });
        return new FormGroup(group);
    }
}
/** @nocollapse */ AttributeControlService.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "13.3.9", ngImport: i0, type: AttributeControlService, deps: [{ token: EntityService }, { token: DomainValueValidator }], target: i0.ɵɵFactoryTarget.Injectable });
/** @nocollapse */ AttributeControlService.ɵprov = i0.ɵɵngDeclareInjectable({ minVersion: "12.0.0", version: "13.3.9", ngImport: i0, type: AttributeControlService, providedIn: 'root' });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "13.3.9", ngImport: i0, type: AttributeControlService, decorators: [{
            type: Injectable,
            args: [{
                    providedIn: 'root'
                }]
        }], ctorParameters: function () { return [{ type: EntityService }, { type: DomainValueValidator }]; } });

/**
 * Convert the date object to a string in format "yyyy-MM-ddThh:mm:ss".
 */
class DatetimeDirective {
    constructor() {
        this.localString = '';
        this.dateChange = new EventEmitter();
    }
    set localDatetime(d) { this.localString = this.toLocalDateString(d); }
    toLocalDateString(timeUTC) {
        if (!timeUTC) {
            return '';
        }
        if (isNaN(timeUTC.getTime())) {
            return '';
        }
        else {
            return timeUTC.getFullYear().toString() + '-' + ('0' + (timeUTC.getMonth() + 1)).slice(-2) + '-'
                + ('0' + (timeUTC.getDate())).slice(-2) + 'T' + timeUTC.toTimeString().slice(0, 8);
        }
    }
    onDateChange(value) {
        if (value !== this.localString) {
            this.localString = value;
            this.dateChange.emit(new Date(value));
        }
    }
}
/** @nocollapse */ DatetimeDirective.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "13.3.9", ngImport: i0, type: DatetimeDirective, deps: [], target: i0.ɵɵFactoryTarget.Directive });
/** @nocollapse */ DatetimeDirective.ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "12.0.0", version: "13.3.9", type: DatetimeDirective, selector: "input[type=datetime-local]", inputs: { localDatetime: "localDatetime" }, outputs: { dateChange: "dateChange" }, host: { listeners: { "change": "onDateChange($event.target.value)" }, properties: { "value": "this.localString" } }, ngImport: i0 });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "13.3.9", ngImport: i0, type: DatetimeDirective, decorators: [{
            type: Directive,
            args: [{
                    // tslint:disable-next-line:directive-selector
                    selector: 'input[type=datetime-local]',
                }]
        }], ctorParameters: function () { return []; }, propDecorators: { localDatetime: [{
                type: Input
            }], dateChange: [{
                type: Output
            }], localString: [{
                type: HostBinding,
                args: ['value']
            }], onDateChange: [{
                type: HostListener,
                args: ['change', ['$event.target.value']]
            }] } });

class AttributeComponent {
    constructor() {
        this.change = new EventEmitter();
    }
    ngOnInit() {
        const fieldCtrl = this.formGroup.get(this.attributeControl.name);
        if (!fieldCtrl) {
            return;
        }
        switch (this.attributeControl.controlType) {
            case 'timestamp':
                if (!this.readonly && !fieldCtrl.value) {
                    this.localDatetime = new Date();
                    fieldCtrl.setValue(this.localDatetime.toISOString().slice(0, 19).replace('T', ' '));
                    fieldCtrl.markAsDirty();
                }
                else if (fieldCtrl.value) {
                    // to support Safari and Firefox, the dateStr format 'yyyy-MM-dd HH:mm:ss' needs to be converted to 'yyyy/MM/dd HH:mm:ss'
                    this.localDatetime = new Date(fieldCtrl.value.replace(/-/g, '/') + ' UTC');
                }
                fieldCtrl.valueChanges.subscribe(value => {
                    if (!value) {
                        return;
                    }
                    this.localDatetime = new Date(value.replace(/-/g, '/') + ' UTC');
                });
                break;
            case 'date':
                if (!this.readonly && !fieldCtrl.value) {
                    this.localDatetime = new Date();
                    fieldCtrl.setValue(this.localDatetime.toISOString().slice(0, 10));
                    fieldCtrl.markAsDirty();
                }
                break;
            default:
        }
        if (this.attributeControl.domainId === 'UUID' && !this.readonly && !fieldCtrl.value) {
            fieldCtrl.setValue(v4());
            fieldCtrl.markAsDirty();
        }
        if (this.attributeControl.domainId === 'GUID' && !this.readonly && !fieldCtrl.value) {
            let guid = v4();
            guid = guid.replace(/-/g, '').toUpperCase();
            fieldCtrl.setValue(guid);
            fieldCtrl.markAsDirty();
        }
    }
    ngOnChanges() {
        if (this.attributeControl.controlType === 'dropdown' ||
            this.attributeControl.controlType === 'checkbox') {
            this.readonly ? this.formGroup.get(this.attributeControl.name).disable() :
                this.formGroup.get(this.attributeControl.name).enable();
        }
    }
    onDateChange(newDate) {
        const UTCDateStr = newDate.toISOString().slice(0, 19).replace('T', ' ');
        const fieldCtrl = this.formGroup.get(this.attributeControl.name);
        if (fieldCtrl) {
            fieldCtrl.setValue(UTCDateStr);
            fieldCtrl.markAsDirty();
        }
    }
    get invalid() {
        var _a;
        return (_a = this.formGroup.controls[this.attributeControl.name]) === null || _a === void 0 ? void 0 : _a.invalid;
    }
    get errorMessage() {
        const fieldCtrl = this.formGroup.get(this.attributeControl.name);
        if (!fieldCtrl) {
            return null;
        }
        if (fieldCtrl.getError('pattern')) {
            return 'The pattern is not correct';
        }
        else if (fieldCtrl.getError('required')) {
            return 'Required';
        }
        else if (fieldCtrl.getError('message')) {
            return fieldCtrl.getError('message');
        }
        else {
            return null;
        }
    }
    get isReadonly() { return this.readonly || this.attributeControl.autoIncrement; }
    onKeyup(attributeName) {
        const fieldCtrl = this.formGroup.get(attributeName);
        fieldCtrl.setValue(fieldCtrl.value.toUpperCase());
    }
    onSearchHelp(attributeControl) {
        if (attributeControl.searchHelpId) {
            this.searchHelpComponent.openSearchHelpModalBySearchHelp(attributeControl.searchHelpId, attributeControl.searchHelpExportField, attributeControl.name, this.formGroup, this.readonly);
        }
        else {
            this.searchHelpComponent.openSearchHelpModalByEntity(attributeControl.domainEntityId, attributeControl.domainRelationId, this.formGroup, this.readonly, attributeControl.name, attributeControl.domainId);
        }
    }
    onChange(event) {
        this.change.emit(event);
    }
}
/** @nocollapse */ AttributeComponent.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "13.3.9", ngImport: i0, type: AttributeComponent, deps: [], target: i0.ɵɵFactoryTarget.Component });
/** @nocollapse */ AttributeComponent.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "12.0.0", version: "13.3.9", type: AttributeComponent, selector: "dk-app-attribute", inputs: { attributeControl: "attributeControl", formGroup: "formGroup", readonly: "readonly", isSmallSize: "isSmallSize", noErrorMsg: "noErrorMsg" }, outputs: { change: "change" }, viewQueries: [{ propertyName: "searchHelpComponent", first: true, predicate: SearchHelpComponent, descendants: true }], usesOnChanges: true, ngImport: i0, template: "<div [formGroup]=\"formGroup\">\n  <div [ngSwitch]=\"attributeControl.controlType\">\n    <div *ngIf=\"attributeControl.controlType === 'text' ||\n                attributeControl.controlType === 'text_capital' ||\n                attributeControl.controlType === 'integer'\" class=\"input-group\">\n      <input *ngSwitchCase=\"'text'\" type=\"text\" class=\"form-control\"\n             [class.form-control-sm] = \"isSmallSize\"\n             [formControlName]=\"attributeControl.name\"\n             [name]=\"attributeControl.key\"\n             [maxlength]=\"attributeControl.maxLength\"\n             [pattern]=\"attributeControl.pattern\"\n             (change) = \"onChange($event)\"\n             [readonly]=\"isReadonly\">\n      <input *ngSwitchCase=\"'text_capital'\" type=\"text\" class=\"form-control\"\n             [class.form-control-sm] = \"isSmallSize\"\n             [formControlName]=\"attributeControl.name\"\n             [name]=\"attributeControl.key\"\n             [maxlength]=\"attributeControl.maxLength\"\n             (keyup)=\"onKeyup(attributeControl.name)\"\n             (change) = \"onChange($event)\"\n             [readonly]=\"isReadonly\" >\n      <input *ngSwitchCase=\"'integer'\" type=\"number\" class=\"form-control\"\n             [class.form-control-sm] = \"isSmallSize\"\n             [formControlName]=\"attributeControl.name\"\n             [name]=\"attributeControl.key\"\n             [pattern]=\"attributeControl.pattern\"\n             (change) = \"onChange($event)\"\n             [readonly]=\"isReadonly\" >\n      <div *ngIf=\"attributeControl.searchHelpId || attributeControl.domainRelationId\" class=\"input-group-append\">\n        <button class=\"btn btn-outline-secondary btn-sm\" type=\"button\" (click)=\"onSearchHelp(attributeControl)\">\n          <span class=\"fas fa-search\"></span>\n        </button>\n      </div>\n    </div>\n    <input *ngSwitchCase=\"'decimal'\" type=\"number\" class=\"form-control\"\n           [class.form-control-sm] = \"isSmallSize\"\n           [formControlName]=\"attributeControl.name\"\n           [name]=\"attributeControl.key\"\n           [step]=\"attributeControl.step\"\n           [placeholder]=\"attributeControl.placeholder\"\n           [pattern]=\"attributeControl.pattern\"\n           (change) = \"onChange($event)\"\n           [readonly]=\"isReadonly\" >\n    <input *ngSwitchCase=\"'checkbox'\" type=\"checkbox\" class=\"mb-1 ml-1\"\n           [class.form-control-sm] = \"isSmallSize\"\n           [formControlName]=\"attributeControl.name\"\n           (change) = \"onChange($event)\"\n           [name]=\"attributeControl.key\">\n    <textarea *ngSwitchCase=\"'textarea'\" class=\"form-control\"\n              [class.form-control-sm] = \"isSmallSize\"\n              [formControlName]=\"attributeControl.name\"\n              [name]=\"attributeControl.key\"\n              [rows]=\"attributeControl.rows\"\n              (change) = \"onChange($event)\"\n              [readonly]=\"isReadonly\"></textarea>\n    <input *ngSwitchCase=\"'file'\" type=\"file\" class=\"form-control\"\n           [class.form-control-sm] = \"isSmallSize\"\n           [formControlName]=\"attributeControl.name\"\n           [name]=\"attributeControl.key\"\n           (change) = \"onChange($event)\"\n           [readonly]=\"isReadonly\" >\n    <input *ngSwitchCase=\"'date'\" type=\"date\" class=\"form-control\"\n           [class.form-control-sm] = \"isSmallSize\"\n           [formControlName]=\"attributeControl.name\"\n           [name]=\"attributeControl.key\"\n           (change) = \"onChange($event)\"\n           [readonly]=\"isReadonly\" >\n    <input *ngSwitchCase=\"'timestamp'\" type=\"datetime-local\" class=\"form-control\" step=\"1\"\n           [class.form-control-sm] = \"isSmallSize\"\n           [(localDatetime)] = \"localDatetime\"\n           (dateChange)=\"onDateChange($event)\"\n           [name]=\"attributeControl.key\"\n           (change) = \"onChange($event)\"\n           [readonly]=\"isReadonly\" >\n    <input *ngSwitchCase=\"'password'\" type=\"password\" class=\"form-control\"\n           [class.form-control-sm] = \"isSmallSize\"\n           [formControlName]=\"attributeControl.name\"\n           [name]=\"attributeControl.key\"\n           (change) = \"onChange($event)\"\n           [readonly]=\"isReadonly\" >\n    <select *ngSwitchCase=\"'dropdown'\" class=\"form-control\"\n            [class.form-control-sm] = \"isSmallSize\"\n            [formControlName]=\"attributeControl.name\"\n            [name]=\"attributeControl.key\"\n            (change) = \"onChange($event)\">\n      <option *ngFor=\"let opt of attributeControl.dropdownList\" [value]=\"opt.key\">{{opt.value}}</option>\n    </select>\n  </div>\n\n  <div *ngIf=\"!noErrorMsg\" [class.ng-invalid]=\"invalid\" class=\"dk-invalid-feedback\">\n    {{errorMessage}}\n  </div>\n</div>\n\n<dk-app-search-help></dk-app-search-help>\n", styles: [""], components: [{ type: SearchHelpComponent, selector: "dk-app-search-help" }], directives: [{ type: i2$1.NgControlStatusGroup, selector: "[formGroupName],[formArrayName],[ngModelGroup],[formGroup],form:not([ngNoForm]),[ngForm]" }, { type: i2$1.FormGroupDirective, selector: "[formGroup]", inputs: ["formGroup"], outputs: ["ngSubmit"], exportAs: ["ngForm"] }, { type: i3.NgSwitch, selector: "[ngSwitch]", inputs: ["ngSwitch"] }, { type: i3.NgIf, selector: "[ngIf]", inputs: ["ngIf", "ngIfThen", "ngIfElse"] }, { type: i3.NgSwitchCase, selector: "[ngSwitchCase]", inputs: ["ngSwitchCase"] }, { type: i2$1.DefaultValueAccessor, selector: "input:not([type=checkbox])[formControlName],textarea[formControlName],input:not([type=checkbox])[formControl],textarea[formControl],input:not([type=checkbox])[ngModel],textarea[ngModel],[ngDefaultControl]" }, { type: i2$1.NgControlStatus, selector: "[formControlName],[ngModel],[formControl]" }, { type: i2$1.FormControlName, selector: "[formControlName]", inputs: ["formControlName", "disabled", "ngModel"], outputs: ["ngModelChange"] }, { type: i2$1.MaxLengthValidator, selector: "[maxlength][formControlName],[maxlength][formControl],[maxlength][ngModel]", inputs: ["maxlength"] }, { type: i2$1.PatternValidator, selector: "[pattern][formControlName],[pattern][formControl],[pattern][ngModel]", inputs: ["pattern"] }, { type: i2$1.NumberValueAccessor, selector: "input[type=number][formControlName],input[type=number][formControl],input[type=number][ngModel]" }, { type: i2$1.CheckboxControlValueAccessor, selector: "input[type=checkbox][formControlName],input[type=checkbox][formControl],input[type=checkbox][ngModel]" }, { type: DatetimeDirective, selector: "input[type=datetime-local]", inputs: ["localDatetime"], outputs: ["dateChange"] }, { type: i2$1.SelectControlValueAccessor, selector: "select:not([multiple])[formControlName],select:not([multiple])[formControl],select:not([multiple])[ngModel]", inputs: ["compareWith"] }, { type: i3.NgForOf, selector: "[ngFor][ngForOf]", inputs: ["ngForOf", "ngForTrackBy", "ngForTemplate"] }, { type: i2$1.NgSelectOption, selector: "option", inputs: ["ngValue", "value"] }, { type: i2$1.ɵNgSelectMultipleOption, selector: "option", inputs: ["ngValue", "value"] }] });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "13.3.9", ngImport: i0, type: AttributeComponent, decorators: [{
            type: Component,
            args: [{ selector: 'dk-app-attribute', template: "<div [formGroup]=\"formGroup\">\n  <div [ngSwitch]=\"attributeControl.controlType\">\n    <div *ngIf=\"attributeControl.controlType === 'text' ||\n                attributeControl.controlType === 'text_capital' ||\n                attributeControl.controlType === 'integer'\" class=\"input-group\">\n      <input *ngSwitchCase=\"'text'\" type=\"text\" class=\"form-control\"\n             [class.form-control-sm] = \"isSmallSize\"\n             [formControlName]=\"attributeControl.name\"\n             [name]=\"attributeControl.key\"\n             [maxlength]=\"attributeControl.maxLength\"\n             [pattern]=\"attributeControl.pattern\"\n             (change) = \"onChange($event)\"\n             [readonly]=\"isReadonly\">\n      <input *ngSwitchCase=\"'text_capital'\" type=\"text\" class=\"form-control\"\n             [class.form-control-sm] = \"isSmallSize\"\n             [formControlName]=\"attributeControl.name\"\n             [name]=\"attributeControl.key\"\n             [maxlength]=\"attributeControl.maxLength\"\n             (keyup)=\"onKeyup(attributeControl.name)\"\n             (change) = \"onChange($event)\"\n             [readonly]=\"isReadonly\" >\n      <input *ngSwitchCase=\"'integer'\" type=\"number\" class=\"form-control\"\n             [class.form-control-sm] = \"isSmallSize\"\n             [formControlName]=\"attributeControl.name\"\n             [name]=\"attributeControl.key\"\n             [pattern]=\"attributeControl.pattern\"\n             (change) = \"onChange($event)\"\n             [readonly]=\"isReadonly\" >\n      <div *ngIf=\"attributeControl.searchHelpId || attributeControl.domainRelationId\" class=\"input-group-append\">\n        <button class=\"btn btn-outline-secondary btn-sm\" type=\"button\" (click)=\"onSearchHelp(attributeControl)\">\n          <span class=\"fas fa-search\"></span>\n        </button>\n      </div>\n    </div>\n    <input *ngSwitchCase=\"'decimal'\" type=\"number\" class=\"form-control\"\n           [class.form-control-sm] = \"isSmallSize\"\n           [formControlName]=\"attributeControl.name\"\n           [name]=\"attributeControl.key\"\n           [step]=\"attributeControl.step\"\n           [placeholder]=\"attributeControl.placeholder\"\n           [pattern]=\"attributeControl.pattern\"\n           (change) = \"onChange($event)\"\n           [readonly]=\"isReadonly\" >\n    <input *ngSwitchCase=\"'checkbox'\" type=\"checkbox\" class=\"mb-1 ml-1\"\n           [class.form-control-sm] = \"isSmallSize\"\n           [formControlName]=\"attributeControl.name\"\n           (change) = \"onChange($event)\"\n           [name]=\"attributeControl.key\">\n    <textarea *ngSwitchCase=\"'textarea'\" class=\"form-control\"\n              [class.form-control-sm] = \"isSmallSize\"\n              [formControlName]=\"attributeControl.name\"\n              [name]=\"attributeControl.key\"\n              [rows]=\"attributeControl.rows\"\n              (change) = \"onChange($event)\"\n              [readonly]=\"isReadonly\"></textarea>\n    <input *ngSwitchCase=\"'file'\" type=\"file\" class=\"form-control\"\n           [class.form-control-sm] = \"isSmallSize\"\n           [formControlName]=\"attributeControl.name\"\n           [name]=\"attributeControl.key\"\n           (change) = \"onChange($event)\"\n           [readonly]=\"isReadonly\" >\n    <input *ngSwitchCase=\"'date'\" type=\"date\" class=\"form-control\"\n           [class.form-control-sm] = \"isSmallSize\"\n           [formControlName]=\"attributeControl.name\"\n           [name]=\"attributeControl.key\"\n           (change) = \"onChange($event)\"\n           [readonly]=\"isReadonly\" >\n    <input *ngSwitchCase=\"'timestamp'\" type=\"datetime-local\" class=\"form-control\" step=\"1\"\n           [class.form-control-sm] = \"isSmallSize\"\n           [(localDatetime)] = \"localDatetime\"\n           (dateChange)=\"onDateChange($event)\"\n           [name]=\"attributeControl.key\"\n           (change) = \"onChange($event)\"\n           [readonly]=\"isReadonly\" >\n    <input *ngSwitchCase=\"'password'\" type=\"password\" class=\"form-control\"\n           [class.form-control-sm] = \"isSmallSize\"\n           [formControlName]=\"attributeControl.name\"\n           [name]=\"attributeControl.key\"\n           (change) = \"onChange($event)\"\n           [readonly]=\"isReadonly\" >\n    <select *ngSwitchCase=\"'dropdown'\" class=\"form-control\"\n            [class.form-control-sm] = \"isSmallSize\"\n            [formControlName]=\"attributeControl.name\"\n            [name]=\"attributeControl.key\"\n            (change) = \"onChange($event)\">\n      <option *ngFor=\"let opt of attributeControl.dropdownList\" [value]=\"opt.key\">{{opt.value}}</option>\n    </select>\n  </div>\n\n  <div *ngIf=\"!noErrorMsg\" [class.ng-invalid]=\"invalid\" class=\"dk-invalid-feedback\">\n    {{errorMessage}}\n  </div>\n</div>\n\n<dk-app-search-help></dk-app-search-help>\n", styles: [""] }]
        }], ctorParameters: function () { return []; }, propDecorators: { attributeControl: [{
                type: Input
            }], formGroup: [{
                type: Input
            }], readonly: [{
                type: Input
            }], isSmallSize: [{
                type: Input
            }], noErrorMsg: [{
                type: Input
            }], change: [{
                type: Output
            }], searchHelpComponent: [{
                type: ViewChild,
                args: [SearchHelpComponent, { static: false }]
            }] } });

class AttributeFormComponent {
    constructor() { }
    ngOnInit() {
    }
}
/** @nocollapse */ AttributeFormComponent.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "13.3.9", ngImport: i0, type: AttributeFormComponent, deps: [], target: i0.ɵɵFactoryTarget.Component });
/** @nocollapse */ AttributeFormComponent.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "12.0.0", version: "13.3.9", type: AttributeFormComponent, selector: "dk-app-attribute-form", inputs: { attributeControl: "attributeControl", formGroup: "formGroup", readonly: "readonly", isSmallSize: "isSmallSize" }, ngImport: i0, template: "<div class=\"form-group row\" [formGroup]=\"formGroup\">\n  <label class=\"col-4 col-form-label text-right\"\n         [class.form-control-sm]=\"isSmallSize\"\n         [attr.for]=\"attributeControl.key\">\n    {{attributeControl.label}}<strong *ngIf=\"attributeControl.primaryKey\" class=\"primaryKey\">*</strong>:\n  </label>\n\n  <dk-app-attribute class=\"col-8\" [attributeControl]=\"attributeControl\" [formGroup]=\"formGroup\"\n                 [isSmallSize]=\"isSmallSize\" [readonly]=\"readonly\">\n  </dk-app-attribute>\n\n</div>\n", styles: [""], components: [{ type: AttributeComponent, selector: "dk-app-attribute", inputs: ["attributeControl", "formGroup", "readonly", "isSmallSize", "noErrorMsg"], outputs: ["change"] }], directives: [{ type: i2$1.NgControlStatusGroup, selector: "[formGroupName],[formArrayName],[ngModelGroup],[formGroup],form:not([ngNoForm]),[ngForm]" }, { type: i2$1.FormGroupDirective, selector: "[formGroup]", inputs: ["formGroup"], outputs: ["ngSubmit"], exportAs: ["ngForm"] }, { type: i3.NgIf, selector: "[ngIf]", inputs: ["ngIf", "ngIfThen", "ngIfElse"] }] });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "13.3.9", ngImport: i0, type: AttributeFormComponent, decorators: [{
            type: Component,
            args: [{ selector: 'dk-app-attribute-form', template: "<div class=\"form-group row\" [formGroup]=\"formGroup\">\n  <label class=\"col-4 col-form-label text-right\"\n         [class.form-control-sm]=\"isSmallSize\"\n         [attr.for]=\"attributeControl.key\">\n    {{attributeControl.label}}<strong *ngIf=\"attributeControl.primaryKey\" class=\"primaryKey\">*</strong>:\n  </label>\n\n  <dk-app-attribute class=\"col-8\" [attributeControl]=\"attributeControl\" [formGroup]=\"formGroup\"\n                 [isSmallSize]=\"isSmallSize\" [readonly]=\"readonly\">\n  </dk-app-attribute>\n\n</div>\n", styles: [""] }]
        }], ctorParameters: function () { return []; }, propDecorators: { attributeControl: [{
                type: Input
            }], formGroup: [{
                type: Input
            }], readonly: [{
                type: Input
            }], isSmallSize: [{
                type: Input
            }] } });

class AttributeForm2Component {
    constructor() { }
    ngOnInit() {
    }
    get invalid() { var _a; return (_a = this.formGroup.controls[this.attributeControl.name]) === null || _a === void 0 ? void 0 : _a.invalid; }
    get errorMessage() {
        const fieldCtrl = this.formGroup.get(this.attributeControl.name);
        if (!fieldCtrl) {
            return null;
        }
        if (fieldCtrl.getError('pattern')) {
            return 'The pattern is not correct';
        }
        else if (fieldCtrl.getError('required')) {
            return 'Required';
        }
        else if (fieldCtrl.getError('message')) {
            return fieldCtrl.getError('message');
        }
        else {
            return null;
        }
    }
}
/** @nocollapse */ AttributeForm2Component.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "13.3.9", ngImport: i0, type: AttributeForm2Component, deps: [], target: i0.ɵɵFactoryTarget.Component });
/** @nocollapse */ AttributeForm2Component.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "12.0.0", version: "13.3.9", type: AttributeForm2Component, selector: "dk-app-attribute-form2", inputs: { attributeControl: "attributeControl", formGroup: "formGroup", readonly: "readonly", isSmallSize: "isSmallSize" }, ngImport: i0, template: "<div class=\"form-group\" [formGroup]=\"formGroup\">\n  <label class=\"col-form-label\"\n         [class.form-control-sm]=\"isSmallSize\"\n         [attr.for]=\"attributeControl.key\">\n    {{attributeControl.label}}<strong *ngIf=\"attributeControl.primaryKey\" class=\"primaryKey\">*</strong>:\n    <span [class.ng-invalid]=\"invalid\" class=\"dk-invalid-feedback\">{{errorMessage}}</span>\n  </label>\n\n  <dk-app-attribute [attributeControl]=\"attributeControl\" [formGroup]=\"formGroup\"\n                    [isSmallSize]=\"isSmallSize\" [readonly]=\"readonly\" [noErrorMsg]=\"true\">\n  </dk-app-attribute>\n\n</div>\n", styles: [""], components: [{ type: AttributeComponent, selector: "dk-app-attribute", inputs: ["attributeControl", "formGroup", "readonly", "isSmallSize", "noErrorMsg"], outputs: ["change"] }], directives: [{ type: i2$1.NgControlStatusGroup, selector: "[formGroupName],[formArrayName],[ngModelGroup],[formGroup],form:not([ngNoForm]),[ngForm]" }, { type: i2$1.FormGroupDirective, selector: "[formGroup]", inputs: ["formGroup"], outputs: ["ngSubmit"], exportAs: ["ngForm"] }, { type: i3.NgIf, selector: "[ngIf]", inputs: ["ngIf", "ngIfThen", "ngIfElse"] }] });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "13.3.9", ngImport: i0, type: AttributeForm2Component, decorators: [{
            type: Component,
            args: [{ selector: 'dk-app-attribute-form2', template: "<div class=\"form-group\" [formGroup]=\"formGroup\">\n  <label class=\"col-form-label\"\n         [class.form-control-sm]=\"isSmallSize\"\n         [attr.for]=\"attributeControl.key\">\n    {{attributeControl.label}}<strong *ngIf=\"attributeControl.primaryKey\" class=\"primaryKey\">*</strong>:\n    <span [class.ng-invalid]=\"invalid\" class=\"dk-invalid-feedback\">{{errorMessage}}</span>\n  </label>\n\n  <dk-app-attribute [attributeControl]=\"attributeControl\" [formGroup]=\"formGroup\"\n                    [isSmallSize]=\"isSmallSize\" [readonly]=\"readonly\" [noErrorMsg]=\"true\">\n  </dk-app-attribute>\n\n</div>\n", styles: [""] }]
        }], ctorParameters: function () { return []; }, propDecorators: { attributeControl: [{
                type: Input
            }], formGroup: [{
                type: Input
            }], readonly: [{
                type: Input
            }], isSmallSize: [{
                type: Input
            }] } });

class AttributeTableComponent {
    constructor() {
        this.isDetailModalShown = false;
        this.isErrorModalShown = false;
    }
    ngOnInit() {
    }
    get displayDetailModal() { return this.isDetailModalShown ? 'block' : 'none'; }
    get displayErrorModal() { return this.isErrorModalShown ? 'block' : 'none'; }
    openDetailModal(index) {
        this.currentFormGroup = this.formArray.at(index);
        this.isDetailModalShown = true;
    }
    closeDetailModal() {
        if (!this.readonly && this.currentFormGroup.dirty) {
            this.currentFormGroup.setValue(this.currentFormGroup.value); // Or the value won't be updated.
        }
        this.isDetailModalShown = false;
    }
    openErrorModal(errorTitle, errorDescription) {
        this.errorTitle = errorTitle;
        this.errorDescription = errorDescription;
        this.isErrorModalShown = true;
    }
    closeErrorModal() {
        this.isErrorModalShown = false;
    }
    deleteRelationInstance(index = 0) {
        switch (this.entityRelation.CARDINALITY) {
            case '[0..n]':
                this.formArray.removeAt(index);
                this.formArray.markAsDirty();
                if (this.formArray.length === 0) {
                    this.entityRelation.EMPTY = true;
                }
                break;
            case '[1..n]':
                if (this.formArray.length === 1) {
                    this.openErrorModal('Deletion Fail', 'The relation requires at lease one entry!');
                }
                else {
                    this.formArray.removeAt(index);
                    this.formArray.markAsDirty();
                }
        }
    }
}
/** @nocollapse */ AttributeTableComponent.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "13.3.9", ngImport: i0, type: AttributeTableComponent, deps: [], target: i0.ɵɵFactoryTarget.Component });
/** @nocollapse */ AttributeTableComponent.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "12.0.0", version: "13.3.9", type: AttributeTableComponent, selector: "dk-app-attribute-table", inputs: { attributeControls: "attributeControls", formArray: "formArray", parentFormGroup: "parentFormGroup", entityRelation: "entityRelation", isSmallSize: "isSmallSize", readonly: "readonly" }, ngImport: i0, template: "<table class=\"table table-bordered relation-table\"\n       [class.table-sm]=\"isSmallSize\"\n       [formGroup]=\"parentFormGroup\">\n  <thead class=\"thead-light\">\n  <tr>\n    <th scope=\"col\">\n      Action\n    </th>\n    <th scope=\"col\" *ngFor=\"let attributeControl of attributeControls\">\n      {{attributeControl.label}}<strong *ngIf=\"attributeControl.primaryKey\" class=\"primaryKey\">*</strong>\n    </th>\n  </tr>\n  </thead>\n\n  <tbody [formArrayName]=\"entityRelation.RELATION_ID\">\n    <tr *ngFor=\"let formGroup of formArray.controls; let i = index\">\n      <td class=\"actions\">\n        <button class=\"btn\" [class.btn-sm]=\"isSmallSize\" type=\"button\" (click)=\"openDetailModal(i)\" title=\"Detail\">\n          <span *ngIf=\"readonly\" class=\"fas fa-search\"></span>\n          <span *ngIf=\"!readonly\" class=\"fas fa-pen\"></span>\n        </button>\n        <button class=\"btn\" [class.btn-sm]=\"isSmallSize\" type=\"button\" [disabled]=\"readonly\" (click)=\"deleteRelationInstance(i)\" title=\"Delete\">\n          <span class=\"far fa-trash-alt\"></span>\n        </button>\n      </td>\n      <td *ngFor=\"let attributeControl of attributeControls\">\n        <dk-app-attribute [attributeControl]=\"attributeControl\" [isSmallSize] = \"isSmallSize\"\n                          [formGroup]=\"formGroup\" [readonly]=\"readonly\">\n\n        </dk-app-attribute>\n      </td>\n    </tr>\n  </tbody>\n</table>\n\n<div *ngIf=\"currentFormGroup\" class=\"modal fade dk-modal-open\" [ngClass]=\"{'show': isDetailModalShown}\"\n     [ngStyle]=\"{'display': displayDetailModal}\" id=\"detailModal\" tabindex=\"-1\" role=\"dialog\">\n  <div class=\"modal-dialog modal-lg\" role=\"document\">\n    <div class=\"modal-content\">\n      <div class=\"modal-header\">\n        <h5 class=\"modal-title\" id=\"detail\">Line Detail</h5>\n        <button type=\"button\" class=\"close\" (click)=\"closeDetailModal()\">\n          <span aria-hidden=\"true\">&times;</span>\n        </button>\n      </div>\n      <div class=\"modal-body\">\n        <dk-app-attribute-form *ngFor=\"let attributeControl of attributeControls\"\n                               [readonly]=\"readonly\" [isSmallSize]=\"isSmallSize\"\n                            [attributeControl]=\"attributeControl\" [formGroup]=\"currentFormGroup\">\n        </dk-app-attribute-form>\n      </div>\n      <div class=\"modal-footer\">\n        <button type=\"button\" class=\"btn btn-primary\" [class.btn-sm]=\"isSmallSize\" (click)=\"closeDetailModal()\">OK</button>\n      </div>\n    </div>\n  </div>\n</div>\n\n<div class=\"modal fade\" [ngClass]=\"{'show': isErrorModalShown}\"\n     [ngStyle]=\"{'display': displayErrorModal}\" id=\"errorModal\" tabindex=\"-1\" role=\"dialog\">\n  <div class=\"modal-dialog\" role=\"document\">\n    <div class=\"modal-content\">\n      <div class=\"modal-header\">\n        <h5 class=\"modal-title\" id=\"error\">{{errorTitle}}</h5>\n        <button type=\"button\" class=\"close\" (click)=\"closeErrorModal()\">\n          <span aria-hidden=\"true\">&times;</span>\n        </button>\n      </div>\n      <div class=\"modal-body\">\n        <p>{{errorDescription}}</p>\n      </div>\n      <div class=\"modal-footer\">\n        <button type=\"button\" class=\"btn btn-primary\" [class.btn-sm]=\"isSmallSize\" (click)=\"closeErrorModal()\">OK</button>\n      </div>\n    </div>\n  </div>\n</div>\n\n", styles: [".relation-table{display:block;overflow-x:auto;white-space:nowrap;border:none}.actions{width:5rem;min-width:5rem;max-width:6rem}.dk-modal-open{overflow:auto}\n"], components: [{ type: AttributeComponent, selector: "dk-app-attribute", inputs: ["attributeControl", "formGroup", "readonly", "isSmallSize", "noErrorMsg"], outputs: ["change"] }, { type: AttributeFormComponent, selector: "dk-app-attribute-form", inputs: ["attributeControl", "formGroup", "readonly", "isSmallSize"] }], directives: [{ type: i2$1.NgControlStatusGroup, selector: "[formGroupName],[formArrayName],[ngModelGroup],[formGroup],form:not([ngNoForm]),[ngForm]" }, { type: i2$1.FormGroupDirective, selector: "[formGroup]", inputs: ["formGroup"], outputs: ["ngSubmit"], exportAs: ["ngForm"] }, { type: i3.NgForOf, selector: "[ngFor][ngForOf]", inputs: ["ngForOf", "ngForTrackBy", "ngForTemplate"] }, { type: i3.NgIf, selector: "[ngIf]", inputs: ["ngIf", "ngIfThen", "ngIfElse"] }, { type: i2$1.FormArrayName, selector: "[formArrayName]", inputs: ["formArrayName"] }, { type: i3.NgClass, selector: "[ngClass]", inputs: ["class", "ngClass"] }, { type: i3.NgStyle, selector: "[ngStyle]", inputs: ["ngStyle"] }] });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "13.3.9", ngImport: i0, type: AttributeTableComponent, decorators: [{
            type: Component,
            args: [{ selector: 'dk-app-attribute-table', template: "<table class=\"table table-bordered relation-table\"\n       [class.table-sm]=\"isSmallSize\"\n       [formGroup]=\"parentFormGroup\">\n  <thead class=\"thead-light\">\n  <tr>\n    <th scope=\"col\">\n      Action\n    </th>\n    <th scope=\"col\" *ngFor=\"let attributeControl of attributeControls\">\n      {{attributeControl.label}}<strong *ngIf=\"attributeControl.primaryKey\" class=\"primaryKey\">*</strong>\n    </th>\n  </tr>\n  </thead>\n\n  <tbody [formArrayName]=\"entityRelation.RELATION_ID\">\n    <tr *ngFor=\"let formGroup of formArray.controls; let i = index\">\n      <td class=\"actions\">\n        <button class=\"btn\" [class.btn-sm]=\"isSmallSize\" type=\"button\" (click)=\"openDetailModal(i)\" title=\"Detail\">\n          <span *ngIf=\"readonly\" class=\"fas fa-search\"></span>\n          <span *ngIf=\"!readonly\" class=\"fas fa-pen\"></span>\n        </button>\n        <button class=\"btn\" [class.btn-sm]=\"isSmallSize\" type=\"button\" [disabled]=\"readonly\" (click)=\"deleteRelationInstance(i)\" title=\"Delete\">\n          <span class=\"far fa-trash-alt\"></span>\n        </button>\n      </td>\n      <td *ngFor=\"let attributeControl of attributeControls\">\n        <dk-app-attribute [attributeControl]=\"attributeControl\" [isSmallSize] = \"isSmallSize\"\n                          [formGroup]=\"formGroup\" [readonly]=\"readonly\">\n\n        </dk-app-attribute>\n      </td>\n    </tr>\n  </tbody>\n</table>\n\n<div *ngIf=\"currentFormGroup\" class=\"modal fade dk-modal-open\" [ngClass]=\"{'show': isDetailModalShown}\"\n     [ngStyle]=\"{'display': displayDetailModal}\" id=\"detailModal\" tabindex=\"-1\" role=\"dialog\">\n  <div class=\"modal-dialog modal-lg\" role=\"document\">\n    <div class=\"modal-content\">\n      <div class=\"modal-header\">\n        <h5 class=\"modal-title\" id=\"detail\">Line Detail</h5>\n        <button type=\"button\" class=\"close\" (click)=\"closeDetailModal()\">\n          <span aria-hidden=\"true\">&times;</span>\n        </button>\n      </div>\n      <div class=\"modal-body\">\n        <dk-app-attribute-form *ngFor=\"let attributeControl of attributeControls\"\n                               [readonly]=\"readonly\" [isSmallSize]=\"isSmallSize\"\n                            [attributeControl]=\"attributeControl\" [formGroup]=\"currentFormGroup\">\n        </dk-app-attribute-form>\n      </div>\n      <div class=\"modal-footer\">\n        <button type=\"button\" class=\"btn btn-primary\" [class.btn-sm]=\"isSmallSize\" (click)=\"closeDetailModal()\">OK</button>\n      </div>\n    </div>\n  </div>\n</div>\n\n<div class=\"modal fade\" [ngClass]=\"{'show': isErrorModalShown}\"\n     [ngStyle]=\"{'display': displayErrorModal}\" id=\"errorModal\" tabindex=\"-1\" role=\"dialog\">\n  <div class=\"modal-dialog\" role=\"document\">\n    <div class=\"modal-content\">\n      <div class=\"modal-header\">\n        <h5 class=\"modal-title\" id=\"error\">{{errorTitle}}</h5>\n        <button type=\"button\" class=\"close\" (click)=\"closeErrorModal()\">\n          <span aria-hidden=\"true\">&times;</span>\n        </button>\n      </div>\n      <div class=\"modal-body\">\n        <p>{{errorDescription}}</p>\n      </div>\n      <div class=\"modal-footer\">\n        <button type=\"button\" class=\"btn btn-primary\" [class.btn-sm]=\"isSmallSize\" (click)=\"closeErrorModal()\">OK</button>\n      </div>\n    </div>\n  </div>\n</div>\n\n", styles: [".relation-table{display:block;overflow-x:auto;white-space:nowrap;border:none}.actions{width:5rem;min-width:5rem;max-width:6rem}.dk-modal-open{overflow:auto}\n"] }]
        }], ctorParameters: function () { return []; }, propDecorators: { attributeControls: [{
                type: Input
            }], formArray: [{
                type: Input
            }], parentFormGroup: [{
                type: Input
            }], entityRelation: [{
                type: Input
            }], isSmallSize: [{
                type: Input
            }], readonly: [{
                type: Input
            }] } });

class JorAngularComponent {
    constructor() { }
    ngOnInit() {
    }
}
/** @nocollapse */ JorAngularComponent.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "13.3.9", ngImport: i0, type: JorAngularComponent, deps: [], target: i0.ɵɵFactoryTarget.Component });
/** @nocollapse */ JorAngularComponent.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "12.0.0", version: "13.3.9", type: JorAngularComponent, selector: "dk-jor-angular", ngImport: i0, template: "<!-- html content -->\n", styles: ["@import\"/src/assets/css/global.css\";\n"], encapsulation: i0.ViewEncapsulation.None });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "13.3.9", ngImport: i0, type: JorAngularComponent, decorators: [{
            type: Component,
            args: [{ selector: 'dk-jor-angular', encapsulation: ViewEncapsulation.None, template: "<!-- html content -->\n", styles: ["@import\"/src/assets/css/global.css\";\n"] }]
        }], ctorParameters: function () { return []; } });

class JorAngularModule {
}
/** @nocollapse */ JorAngularModule.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "13.3.9", ngImport: i0, type: JorAngularModule, deps: [], target: i0.ɵɵFactoryTarget.NgModule });
/** @nocollapse */ JorAngularModule.ɵmod = i0.ɵɵngDeclareNgModule({ minVersion: "12.0.0", version: "13.3.9", ngImport: i0, type: JorAngularModule, declarations: [JorAngularComponent,
        SearchHelpComponent,
        AttributeComponent,
        AttributeFormComponent,
        AttributeTableComponent,
        AttributeForm2Component,
        DatetimeDirective], imports: [CommonModule,
        HttpClientModule,
        FormsModule,
        ReactiveFormsModule], exports: [SearchHelpComponent,
        AttributeComponent,
        AttributeFormComponent,
        AttributeForm2Component,
        AttributeTableComponent] });
/** @nocollapse */ JorAngularModule.ɵinj = i0.ɵɵngDeclareInjector({ minVersion: "12.0.0", version: "13.3.9", ngImport: i0, type: JorAngularModule, imports: [[
            CommonModule,
            HttpClientModule,
            FormsModule,
            ReactiveFormsModule
        ]] });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "13.3.9", ngImport: i0, type: JorAngularModule, decorators: [{
            type: NgModule,
            args: [{
                    declarations: [
                        JorAngularComponent,
                        SearchHelpComponent,
                        AttributeComponent,
                        AttributeFormComponent,
                        AttributeTableComponent,
                        AttributeForm2Component,
                        DatetimeDirective
                    ],
                    imports: [
                        CommonModule,
                        HttpClientModule,
                        FormsModule,
                        ReactiveFormsModule
                    ],
                    exports: [
                        SearchHelpComponent,
                        AttributeComponent,
                        AttributeFormComponent,
                        AttributeForm2Component,
                        AttributeTableComponent
                    ]
                }]
        }] });

/*
 * Public API Surface of jor-angular
 */

/**
 * Generated bundle index. Do not edit.
 */

export { Association, Attribute, AttributeBase, AttributeComponent, AttributeControlService, AttributeForm2Component, AttributeFormComponent, AttributeTableComponent, DataDomainH, DataDomainMeta, DataDomainValue, DataElementH, DataElementMeta, Entity, EntityMeta, EntityRelation, EntityService, EntityType, FieldsMappingPair, Involve, JorAngularModule, PartnerInstance, PartnerRole, Projection, QueryObject, Relation, RelationMeta, Relationship, RelationshipH, RelationshipInstance, RelationshipMeta, Role, RoleH, RoleMeta, RoleRelation, SearchHelp, SearchHelpComponent, SearchHelpField, SearchHelpFieldMeta, SearchHelpH, SearchHelpMeta, Selection, Sort, UiMapperService };
//# sourceMappingURL=jor-angular.mjs.map
